@@ -1,3 +1,7 @@
+# SPDX-License-Identifier: Apache-2.0
+# SPDX-FileCopyrightText: 2021 The Elixir Team
+# SPDX-FileCopyrightText: 2012 Plataformatec
+
 Code.require_file("test_helper.exs", __DIR__)
 
 require EEx
@@ -198,15 +202,6 @@ defmodule EExTest do
       assert_eval("foo baz", "foo <%= if false do %>bar<% else %>baz<% end %>")
     end
 
-    test "embedded code with comments in do end" do
-      assert_eval("foo bar", "foo <%= case true do %><%# comment %><% true -> %>bar<% end %>")
-
-      assert_eval(
-        "foo\n\nbar\n",
-        "foo\n<%= case true do %>\n<%# comment %>\n<% true -> %>\nbar\n<% end %>"
-      )
-    end
-
     test "embedded code with multi-line comments in do end" do
       assert_eval("foo bar", "foo <%= case true do %><%!-- comment --%><% true -> %>bar<% end %>")
 
@@ -507,6 +502,59 @@ defmodule EExTest do
       end
     end
 
+    test "from Elixir parser" do
+      line = __ENV__.line + 6
+
+      message =
+        assert_raise TokenMissingError, fn ->
+          EEx.compile_string(
+            """
+            <li>
+              <strong>Some:</strong>
+              <%= true && @some[ %>
+            </li>
+            """,
+            file: __ENV__.file,
+            line: line,
+            indentation: 12
+          )
+        end
+
+      assert message |> Exception.message() |> strip_ansi() =~ """
+                  │
+              514 │                   true && @some[\s
+                  │                                │ └ missing closing delimiter (expected "]")
+                  │                                └ unclosed delimiter
+             """
+    end
+
+    test "from Elixir parser with line breaks" do
+      line = __ENV__.line + 6
+
+      message =
+        assert_raise TokenMissingError, fn ->
+          EEx.compile_string(
+            """
+            <li>
+              <strong>Some:</strong>
+              <%= true &&
+                @some[ %>
+            </li>
+            """,
+            file: __ENV__.file,
+            line: line,
+            indentation: 12
+          )
+        end
+
+      assert message |> Exception.message() |> strip_ansi() =~ """
+                  │
+              #{line + 3} │                 @some[\s
+                  │                      │ └ missing closing delimiter (expected "]")
+                  │                      └ unclosed delimiter
+             """
+    end
+
     test "honor line numbers" do
       assert_raise EEx.SyntaxError,
                    "nofile:100:6: expected closing '%>' for EEx expression",
@@ -869,13 +917,13 @@ defmodule EExTest do
       file = to_charlist(Path.relative_to_cwd(__ENV__.file))
 
       assert EExTest.Compiled.before_compile() ==
-               {7, {EExTest.Compiled, :before_compile, 0, [file: file, line: 7]}}
+               {11, {EExTest.Compiled, :before_compile, 0, [file: file, line: 11]}}
 
       assert EExTest.Compiled.after_compile() ==
-               {21, {EExTest.Compiled, :after_compile, 0, [file: file, line: 21]}}
+               {25, {EExTest.Compiled, :after_compile, 0, [file: file, line: 25]}}
 
       assert EExTest.Compiled.unknown() ==
-               {26, {EExTest.Compiled, :unknown, 0, [file: ~c"unknown", line: 26]}}
+               {30, {EExTest.Compiled, :unknown, 0, [file: ~c"unknown", line: 30]}}
     end
   end
 
@@ -953,6 +1001,12 @@ defmodule EExTest do
     end
   end
 
+  @strip_ansi [IO.ANSI.green(), IO.ANSI.red(), IO.ANSI.reset()]
+
+  defp strip_ansi(doc) do
+    String.replace(doc, @strip_ansi, "")
+  end
+
   defp assert_eval(expected, actual, binding \\ [], opts \\ []) do
     opts = Keyword.merge([file: __ENV__.file, engine: opts[:engine] || EEx.Engine], opts)
     result = EEx.eval_string(actual, binding, opts)
