@@ -1,3 +1,7 @@
+# SPDX-License-Identifier: Apache-2.0
+# SPDX-FileCopyrightText: 2021 The Elixir Team
+# SPDX-FileCopyrightText: 2012 Plataformatec
+
 import Kernel, except: [inspect: 1]
 import Inspect.Algebra
 
@@ -60,9 +64,11 @@ defprotocol Inspect do
 
     * `:except` - remove the given fields when inspecting.
 
-    * `:optional` - (since v1.14.0) do not include a field if it
-      matches its default value. This can be used to simplify the
-      struct representation at the cost of hiding information.
+    * `:optional` - (since v1.14.0) a list of fields that should not be
+      included when they match their default value. This can be used to
+      simplify the struct representation at the cost of hiding
+      information. Since v1.19.0, the `:all` atom can be passed to
+      mark all fields as optional.
 
   Whenever `:only` or `:except` are used to restrict fields,
   the struct will be printed using the `#User<...>` notation,
@@ -78,15 +84,25 @@ defprotocol Inspect do
       #=> #User<id: 1, name: "Jane", ...>
 
   If you use only the `:optional` option, the struct will still be
-  printed as `%User{...}`.
+  printed as a valid struct.
+
+      defmodule Point do
+        @derive {Inspect, optional: [:z]}
+        defstruct [x: 0, y: 0, z: 0]
+      end
+
+      inspect(%Point{x: 1})
+      %Point{x: 1, y: 0}
 
   ## Custom implementation
 
   You can also define your custom protocol implementation by
   defining the `inspect/2` function. The function receives the
   entity to be inspected followed by the inspecting options,
-  represented by the struct `Inspect.Opts`. Building of the
-  algebra document is done with `Inspect.Algebra`.
+  represented by the struct `Inspect.Opts` and it must return
+  an algebra document alongside the updated options (or, optionally,
+  just the algebra document). Building of the algebra document
+  is done with `Inspect.Algebra`.
 
   Many times, inspecting a structure can be implemented in function
   of existing entities. For example, here is `MapSet`'s `inspect/2`
@@ -96,16 +112,19 @@ defprotocol Inspect do
         import Inspect.Algebra
 
         def inspect(map_set, opts) do
-          concat(["MapSet.new(", Inspect.List.inspect(MapSet.to_list(map_set), opts), ")"])
+          {doc, opts} = to_doc_with_opts(MapSet.to_list(map_set), opts)
+          {concat(["MapSet.new(", doc, ")"]), opts}
         end
       end
 
-  The [`concat/1`](`Inspect.Algebra.concat/1`) function comes from
-  `Inspect.Algebra` and it concatenates algebra documents together.
+  First [`to_doc_with_opts/2`](`Inspect.Algebra.to_doc_with_opts/2`) is
+  used to convert another data structure into its algebra document and
+  then [`concat/1`](`Inspect.Algebra.concat/1`) concatenates algebra
+  documents together.
+
   In the example above it is concatenating the string `"MapSet.new("`,
-  the document returned by `Inspect.Algebra.to_doc/2`, and the final
-  string `")"`. Therefore, the MapSet with the numbers 1, 2, and 3
-  will be printed as:
+  the document returned by `to_doc_with_opts/2`, and the final string `")"`.
+  Therefore, the MapSet with the numbers 1, 2, and 3 will be printed as:
 
       iex> MapSet.new([1, 2, 3], fn x -> x * 2 end)
       MapSet.new([2, 4, 6])
@@ -134,6 +153,9 @@ defprotocol Inspect do
 
           Inspect.MapSet.inspect(MapSet.new(), %Inspect.Opts{})
 
+      Note that, from Elixir v1.19, the inspect protocol was augmented to
+      allow a two-element tuple with the document and the updated options
+      to be returned from the protocol.
   """
 
   # Handle structs in Any
@@ -146,11 +168,19 @@ defprotocol Inspect do
 
     only = Keyword.get(options, :only, fields)
     except = Keyword.get(options, :except, [])
-    optional = Keyword.get(options, :optional, [])
 
     :ok = validate_option(:only, only, fields, module)
     :ok = validate_option(:except, except, fields, module)
-    :ok = validate_option(:optional, optional, fields, module)
+
+    optional =
+      case Keyword.get(options, :optional, []) do
+        :all ->
+          fields
+
+        optional ->
+          :ok = validate_option(:optional, optional, fields, module)
+          optional
+      end
 
     inspect_module =
       if fields == Enum.sort(only) and except == [] do
@@ -200,13 +230,26 @@ defprotocol Inspect do
                 do: var!(info)
 
           var!(name) = Macro.inspect_atom(:literal, unquote(module))
-          unquote(inspect_module).inspect(var!(struct), var!(name), var!(infos), var!(opts))
+
+          unquote(inspect_module).inspect_as_struct(
+            var!(struct),
+            var!(name),
+            var!(infos),
+            var!(opts)
+          )
         end
       end
     end
   end
 
   defp validate_option(option, option_list, fields, module) do
+    if not is_list(option_list) do
+      raise ArgumentError,
+            "invalid value #{Kernel.inspect(option_list)} in #{Kernel.inspect(option)} " <>
+              "when deriving the Inspect protocol for #{Kernel.inspect(module)} " <>
+              "(expected a list)"
+    end
+
     case option_list -- fields do
       [] ->
         :ok
@@ -226,7 +269,8 @@ defprotocol Inspect do
   `Inspect.Algebra.to_doc/2` should be preferred as it handles structs
   and exceptions.
   """
-  @spec inspect(t, Inspect.Opts.t()) :: Inspect.Algebra.t()
+  @spec inspect(t, Inspect.Opts.t()) ::
+          Inspect.Algebra.t() | {Inspect.Algebra.t(), Inspect.Opts.t()}
   def inspect(term, opts)
 end
 
@@ -268,11 +312,13 @@ defimpl Inspect, for: BitString do
     color_doc("<<>>", :binary, opts)
   end
 
-  defp inspect_bitstring(bitstring, opts) do
+  defp inspect_bitstring(bitstring, %{limit: limit} = opts) do
     left = color_doc("<<", :binary, opts)
     right = color_doc(">>", :binary, opts)
-    inner = each_bit(bitstring, opts.limit, opts)
-    group(concat(concat(left, nest(inner, 2)), right))
+    inner = each_bit(bitstring, limit, opts)
+    doc = group(concat(concat(left, nest(inner, 2)), right))
+    new_limit = if limit == :infinity, do: limit, else: max(0, limit - byte_size(bitstring))
+    {doc, %{opts | limit: new_limit}}
   end
 
   defp each_bit(_, 0, _) do
@@ -280,7 +326,7 @@ defimpl Inspect, for: BitString do
   end
 
   defp each_bit(<<>>, _counter, _opts) do
-    :doc_nil
+    Inspect.Algebra.empty()
   end
 
   defp each_bit(<<h::8>>, _counter, opts) do
@@ -353,17 +399,21 @@ defimpl Inspect, for: List do
         color_doc(IO.iodata_to_binary(inspected), :charlist, opts)
 
       keyword?(term) ->
-        container_doc(open, term, close, opts, &keyword/2, separator: sep, break: :strict)
+        container_doc_with_opts(open, term, close, opts, &keyword/2,
+          separator: sep,
+          break: :strict
+        )
 
       true ->
-        container_doc(open, term, close, opts, &to_doc/2, separator: sep)
+        container_doc_with_opts(open, term, close, opts, &to_doc_with_opts/2, separator: sep)
     end
   end
 
   @doc false
   def keyword({key, value}, opts) do
     key = color_doc(Macro.inspect_atom(:key, key), :atom, opts)
-    concat(key, concat(" ", to_doc(value, opts)))
+    {doc, opts} = to_doc_with_opts(value, opts)
+    {concat(key, concat(" ", doc)), opts}
   end
 
   @doc false
@@ -384,12 +434,24 @@ defimpl Inspect, for: Tuple do
     sep = color_doc(",", :tuple, opts)
     close = color_doc("}", :tuple, opts)
     container_opts = [separator: sep, break: :flex]
-    container_doc(open, Tuple.to_list(tuple), close, opts, &to_doc/2, container_opts)
+
+    container_doc_with_opts(
+      open,
+      Tuple.to_list(tuple),
+      close,
+      opts,
+      &to_doc_with_opts/2,
+      container_opts
+    )
   end
 end
 
 defimpl Inspect, for: Map do
   def inspect(map, opts) do
+    inspect_as_map(map, opts)
+  end
+
+  def inspect_as_map(map, opts) do
     list =
       if Keyword.get(opts.custom_options, :sort_maps) do
         map |> Map.to_list() |> :lists.sort()
@@ -408,20 +470,22 @@ defimpl Inspect, for: Map do
     map_container_doc(list, "", opts, fun)
   end
 
-  def inspect(map, name, infos, opts) do
+  def inspect_as_struct(map, name, infos, opts) do
     fun = fn %{field: field}, opts -> Inspect.List.keyword({field, Map.get(map, field)}, opts) end
     map_container_doc(infos, name, opts, fun)
   end
 
   defp to_assoc({key, value}, opts, sep) do
-    concat(concat(to_doc(key, opts), sep), to_doc(value, opts))
+    {key_doc, opts} = to_doc_with_opts(key, opts)
+    {value_doc, opts} = to_doc_with_opts(value, opts)
+    {concat(concat(key_doc, sep), value_doc), opts}
   end
 
   defp map_container_doc(list, name, opts, fun) do
     open = color_doc("%" <> name <> "{", :map, opts)
     sep = color_doc(",", :map, opts)
     close = color_doc("}", :map, opts)
-    container_doc(open, list, close, opts, fun, separator: sep, break: :strict)
+    container_doc_with_opts(open, list, close, opts, fun, separator: sep, break: :strict)
   end
 end
 
@@ -479,9 +543,9 @@ defimpl Inspect, for: Regex do
       :error ->
         concat([
           "Regex.compile!(",
-          Inspect.BitString.inspect(regex.source, opts),
+          to_doc(regex.source, opts),
           ", ",
-          Inspect.List.inspect(regex_opts, opts),
+          to_doc(regex_opts, opts),
           ")"
         ])
 
@@ -577,36 +641,6 @@ defimpl Inspect, for: Function do
   end
 end
 
-defimpl Inspect, for: Inspect.Error do
-  @impl true
-  def inspect(%{stacktrace: stacktrace} = inspect_error, _opts) do
-    message = Exception.message(inspect_error)
-    format_output(message, stacktrace)
-  end
-
-  defp format_output(message, [_ | _] = stacktrace) do
-    stacktrace = Exception.format_stacktrace(stacktrace)
-
-    """
-    #Inspect.Error<
-    #{Inspect.Error.pad(message, 2)}
-
-      Stacktrace:
-
-    #{stacktrace}
-    >\
-    """
-  end
-
-  defp format_output(message, []) do
-    """
-    #Inspect.Error<
-      #{Inspect.Error.pad(message, 2)}
-    >\
-    """
-  end
-end
-
 defimpl Inspect, for: PID do
   def inspect(pid, _opts) do
     "#PID" <> IO.iodata_to_binary(:erlang.pid_to_list(pid))
@@ -628,26 +662,15 @@ end
 
 defimpl Inspect, for: Any do
   def inspect(%module{} = struct, opts) do
-    try do
-      {module.__struct__(), module.__info__(:struct)}
-    rescue
-      _ -> Inspect.Map.inspect(struct, opts)
-    else
-      {dunder, fields} ->
-        if Map.keys(dunder) == Map.keys(struct) do
-          infos =
-            for %{field: field} = info <- fields,
-                field not in [:__struct__, :__exception__],
-                do: info
+    info =
+      for %{field: field} = map <- module.__info__(:struct),
+          field != :__exception__,
+          do: map
 
-          Inspect.Map.inspect(struct, Macro.inspect_atom(:literal, module), infos, opts)
-        else
-          Inspect.Map.inspect(struct, opts)
-        end
-    end
+    Inspect.Map.inspect_as_struct(struct, Macro.inspect_atom(:literal, module), info, opts)
   end
 
-  def inspect(map, name, infos, opts) do
+  def inspect_as_struct(map, name, infos, opts) do
     open = color_doc("#" <> name <> "<", :map, opts)
     sep = color_doc(",", :map, opts)
     close = color_doc(">", :map, opts)
@@ -661,6 +684,25 @@ defimpl Inspect, for: Any do
   end
 end
 
+defimpl Inspect, for: Range do
+  import Inspect.Algebra
+  import Kernel, except: [inspect: 2]
+
+  def inspect(first..last//1, opts) when last >= first do
+    concat([to_doc(first, opts), "..", to_doc(last, opts)])
+  end
+
+  def inspect(first..last//step, opts) do
+    concat([to_doc(first, opts), "..", to_doc(last, opts), "//", to_doc(step, opts)])
+  end
+
+  # TODO: Remove me on v2.0
+  def inspect(%{__struct__: Range, first: first, last: last} = range, opts) do
+    step = if first <= last, do: 1, else: -1
+    inspect(Map.put(range, :step, step), opts)
+  end
+end
+
 require Protocol
 
 Protocol.derive(
