@@ -1,3 +1,7 @@
+# SPDX-License-Identifier: Apache-2.0
+# SPDX-FileCopyrightText: 2021 The Elixir Team
+# SPDX-FileCopyrightText: 2012 Plataformatec
+
 defmodule Keyword do
   @moduledoc """
   A keyword list is a list that consists exclusively of two-element tuples.
@@ -6,7 +10,7 @@ defmodule Keyword do
   The second element, known as the *value*, can be any term.
 
   Keywords are mostly used to work with optional values. For a general introduction
-  to keywords and how the compare with maps, see our [Keyword and Maps](keywords-and-maps.md)
+  to keywords and how they compare with maps, see our [Keyword and Maps](keywords-and-maps.md)
   guide.
 
   ## Examples
@@ -525,7 +529,8 @@ defmodule Keyword do
       iex> Keyword.get_and_update!([a: 1], :b, fn current_value ->
       ...>   {current_value, "new value!"}
       ...> end)
-      ** (KeyError) key :b not found in: [a: 1]
+      ** (KeyError) key :b not found in:
+      ...
 
       iex> Keyword.get_and_update!([a: 1], :a, fn _ ->
       ...>   :pop
@@ -592,7 +597,8 @@ defmodule Keyword do
       iex> Keyword.fetch!([a: 1], :a)
       1
       iex> Keyword.fetch!([a: 1], :b)
-      ** (KeyError) key :b not found in: [a: 1]
+      ** (KeyError) key :b not found in:
+      ...
 
   """
   @spec fetch!(t, key) :: value
@@ -736,6 +742,10 @@ defmodule Keyword do
 
       iex> Keyword.delete_first([a: 1, b: 2, a: 3], :a)
       [b: 2, a: 3]
+
+      iex> Keyword.delete_first([a: 1, b: 2, b: 3], :b)
+      [a: 1, b: 3]
+
       iex> Keyword.delete_first([b: 2], :a)
       [b: 2]
 
@@ -756,10 +766,6 @@ defmodule Keyword do
     [pair | delete_first_key(tail, key)]
   end
 
-  defp delete_first_key([], _key) do
-    []
-  end
-
   @doc """
   Puts the given `value` under the specified `key`.
 
@@ -875,7 +881,8 @@ defmodule Keyword do
       [a: 1, b: :new, c: 3]
 
       iex> Keyword.replace!([a: 1], :b, 2)
-      ** (KeyError) key :b not found in: [a: 1]
+      ** (KeyError) key :b not found in:
+      ...
 
   """
   @doc since: "1.5.0"
@@ -1131,7 +1138,8 @@ defmodule Keyword do
       [a: 1, b: 4, c: 3]
 
       iex> Keyword.update!([a: 1], :b, &(&1 * 2))
-      ** (KeyError) key :b not found in: [a: 1]
+      ** (KeyError) key :b not found in:
+      ...
 
   """
   @spec update!(t, key, (current_value :: value -> new_value :: value)) :: t
@@ -1344,7 +1352,8 @@ defmodule Keyword do
       iex> Keyword.pop!([a: 1, a: 2], :a)
       {1, []}
       iex> Keyword.pop!([a: 1], :b)
-      ** (KeyError) key :b not found in: [a: 1]
+      ** (KeyError) key :b not found in:
+      ...
 
   """
   @doc since: "1.10.0"
