@@ -1,3 +1,7 @@
+# SPDX-License-Identifier: Apache-2.0
+# SPDX-FileCopyrightText: 2021 The Elixir Team
+# SPDX-FileCopyrightText: 2012 Plataformatec
+
 defmodule Supervisor do
   @moduledoc ~S"""
   A behaviour module for implementing supervisors.
@@ -130,11 +134,11 @@ defmodule Supervisor do
       to start the child process. This key is required.
 
     * `:restart` - an atom that defines when a terminated child process
-       should be restarted (see the "Restart values" section below).
+       should be restarted (see the ["Restart values"](#module-restart-values-restart) section below).
        This key is optional and defaults to `:permanent`.
 
     * `:shutdown` - an integer or atom that defines how a child process should
-      be terminated (see the "Shutdown values" section below). This key
+      be terminated (see the ["Shutdown values"](#module-shutdown-values-shutdown) section below). This key
       is optional and defaults to `5_000` if the type is `:worker` or
       `:infinity` if the type is `:supervisor`.
 
@@ -525,7 +529,7 @@ defmodule Supervisor do
       import Supervisor.Spec
       @behaviour Supervisor
 
-      unless Module.has_attribute?(__MODULE__, :doc) do
+      if not Module.has_attribute?(__MODULE__, :doc) do
         @doc """
         Returns a specification to start this module under a supervisor.
 
@@ -595,7 +599,7 @@ defmodule Supervisor do
   @typedoc "The supervisor reference."
   @type supervisor :: pid | name | {atom, node}
 
-  @typedoc "Options given to `start_link/2` and `c:init/1`."
+  @typedoc "Options given to `start_link/2` and `init/2`."
   @type init_option ::
           {:strategy, strategy}
           | {:max_restarts, non_neg_integer}
@@ -655,6 +659,19 @@ defmodule Supervisor do
   @typedoc since: "1.16.0"
   @type module_spec :: {module(), args :: term()} | module()
 
+  @typedoc """
+  Options for overriding child specification fields.
+  """
+  @type child_spec_overrides :: [
+          id: atom() | term(),
+          start: {module(), atom(), [term()]},
+          restart: restart(),
+          shutdown: shutdown(),
+          type: type(),
+          modules: [module()] | :dynamic,
+          significant: boolean()
+        ]
+
   @doc """
   Starts a supervisor with the given children.
 
@@ -892,7 +909,7 @@ defmodule Supervisor do
       #=>   start: {Agent, :start_link, [fn -> :ok end]}}
 
   """
-  @spec child_spec(child_spec() | module_spec(), keyword()) :: child_spec()
+  @spec child_spec(child_spec() | module_spec(), child_spec_overrides()) :: child_spec()
   def child_spec(module_or_map, overrides)
 
   def child_spec({_, _, _, _, _, _} = tuple, _overrides) do
@@ -986,6 +1003,12 @@ defmodule Supervisor do
   value, or if it fails, the child specification is discarded and this function
   returns `{:error, error}` where `error` is a term containing information about
   the error and child specification.
+
+  > #### Order Among Children {: .tip}
+  >
+  > The child specification is **appended** to the children of `supervisor`.
+  > This guarantees that semantics of things such as the `:rest_for_one` strategy
+  > are preserved correctly.
   """
   @spec start_child(
           supervisor,
