@@ -5,11 +5,14 @@
 %% Compiler backend to Erlang.
 
 -module(elixir_erl).
--export([elixir_to_erl/1, elixir_to_erl/2, definition_to_anonymous/5, compile/1, consolidate/3,
-         get_ann/1, debug_info/4, scope/2, format_error/1]).
+-export([elixir_to_erl/1, elixir_to_erl/2, definition_to_anonymous/5, compile/2, consolidate/4,
+         get_ann/1, debug_info/4, scope/2, checker_version/0, format_error/1]).
 -include("elixir.hrl").
 -define(typespecs, 'Elixir.Kernel.Typespec').
 
+checker_version() ->
+  elixir_checker_v3.
+
 %% debug_info callback
 
 debug_info(elixir_v1, _Module, none, _Opts) ->
@@ -132,14 +135,14 @@ scope(_Meta, ExpandCaptures) ->
 
 %% Static compilation hook, used in protocol consolidation
 
-consolidate(Map, TypeSpecs, DocsChunk) ->
-  {Prefix, Forms, Def, _Defmacro, _Macros} = dynamic_form(Map, nil),
-  CheckerChunk = checker_chunk(Map, Def, chunk_opts(Map)),
+consolidate(Map, Checker, TypeSpecs, DocsChunk) ->
+  {Prefix, Forms, _Def, _Defmacro, _Macros} = dynamic_form(Map, nil),
+  CheckerChunk = checker_chunk(Checker, chunk_opts(Map)),
   load_form(Map, Prefix, Forms, TypeSpecs, DocsChunk ++ CheckerChunk).
 
 %% Dynamic compilation hook, used in regular compiler
 
-compile(#{module := Module, anno := Anno} = BaseMap) ->
+compile(#{module := Module, anno := Anno} = BaseMap, Signatures) ->
   Map =
     case elixir_erl_compiler:env_compiler_options() of
       [] -> BaseMap;
@@ -161,7 +164,7 @@ compile(#{module := Module, anno := Anno} = BaseMap) ->
 
   ChunkOpts = chunk_opts(Map),
   DocsChunk = docs_chunk(Map, Set, Module, Anno, Def, Defmacro, Types, Callbacks, ChunkOpts),
-  CheckerChunk = checker_chunk(Map, Def, ChunkOpts),
+  CheckerChunk = checker_chunk(Map, Def, Signatures, ChunkOpts),
   load_form(Map, Prefix, Forms, TypeSpecs, DocsChunk ++ CheckerChunk).
 
 chunk_opts(Map) ->
@@ -639,9 +642,13 @@ signature_to_binary(_, Name, Signature) ->
   Doc = 'Elixir.Inspect.Algebra':format('Elixir.Code':quoted_to_algebra(Quoted), infinity),
   'Elixir.IO':iodata_to_binary(Doc).
 
-checker_chunk(Map, Def, ChunkOpts) ->
-  #{deprecated := Deprecated, defines_behaviour := DefinesBehaviour,
-    signatures := Signatures, attributes := Attributes} = Map,
+checker_chunk(nil, _ChunkOpts) ->
+  [];
+checker_chunk(Contents, ChunkOpts) ->
+  [{<<"ExCk">>, term_to_binary({checker_version(), Contents}, ChunkOpts)}].
+
+checker_chunk(Map, Def, Signatures, ChunkOpts) ->
+  #{deprecated := Deprecated, defines_behaviour := DefinesBehaviour, attributes := Attributes} = Map,
   DeprecatedMap = maps:from_list(Deprecated),
 
   Exports =
@@ -662,7 +669,7 @@ checker_chunk(Map, Def, ChunkOpts) ->
     end
   },
 
-  [{<<"ExCk">>, term_to_binary({elixir_checker_v3, Contents}, ChunkOpts)}].
+  checker_chunk(Contents, ChunkOpts).
 
 prepend_behaviour_info(true, Def) -> [{{behaviour_info, 1}, []} | Def];
 prepend_behaviour_info(false, Def) -> Def.
