@@ -1,3 +1,6 @@
+# SPDX-License-Identifier: Apache-2.0
+# SPDX-FileCopyrightText: 2021 The Elixir Team
+
 Code.require_file("test_helper.exs", __DIR__)
 
 defmodule CodeFragmentTest do
@@ -124,8 +127,6 @@ defmodule CodeFragmentTest do
       assert CF.cursor_context("hello(\t") == {:local_call, ~c"hello"}
       assert CF.cursor_context("hello(\n") == {:local_call, ~c"hello"}
       assert CF.cursor_context("hello(\r\n") == {:local_call, ~c"hello"}
-      assert CF.cursor_context("...(") == {:local_call, ~c"..."}
-      assert CF.cursor_context("...(\s") == {:local_call, ~c"..."}
     end
 
     test "dot_arity" do
@@ -237,6 +238,9 @@ defmodule CodeFragmentTest do
       assert CF.cursor_context("Hello::Wor") == {:alias, ~c"Wor"}
       assert CF.cursor_context("Hello..Wor") == {:alias, ~c"Wor"}
 
+      assert CF.cursor_context("hello.World") ==
+               {:alias, {:local_or_var, ~c"hello"}, ~c"World"}
+
       assert CF.cursor_context("__MODULE__.Wor") ==
                {:alias, {:local_or_var, ~c"__MODULE__"}, ~c"Wor"}
 
@@ -298,6 +302,7 @@ defmodule CodeFragmentTest do
     end
 
     test "operators" do
+      assert CF.cursor_context("/") == {:operator, ~c"/"}
       assert CF.cursor_context("+") == {:operator, ~c"+"}
       assert CF.cursor_context("++") == {:operator, ~c"++"}
       assert CF.cursor_context("!") == {:operator, ~c"!"}
@@ -320,6 +325,11 @@ defmodule CodeFragmentTest do
       assert CF.cursor_context("<~> ") == {:operator_call, ~c"<~>"}
       assert CF.cursor_context(":: ") == {:operator_call, ~c"::"}
 
+      assert CF.cursor_context("...(") == {:operator_call, ~c"..."}
+      assert CF.cursor_context("...(\s") == {:operator_call, ~c"..."}
+      assert CF.cursor_context("+(") == {:operator_call, ~c"+"}
+      assert CF.cursor_context("++(\s") == {:operator_call, ~c"++"}
+
       assert CF.cursor_context("+/") == {:operator_arity, ~c"+"}
       assert CF.cursor_context("++/") == {:operator_arity, ~c"++"}
       assert CF.cursor_context("!/") == {:operator_arity, ~c"!"}
@@ -352,6 +362,15 @@ defmodule CodeFragmentTest do
       assert CF.cursor_context("~r/") == :none
       assert CF.cursor_context("~r<") == :none
 
+      assert CF.cursor_context("~r''") == :none
+      assert CF.cursor_context("~r' '") == :none
+      assert CF.cursor_context("~r'foo'") == :none
+
+      # The slash is used in sigils, arities, and operators, so there is ambiguity
+      assert CF.cursor_context("~r//") == {:operator, ~c"/"}
+      assert CF.cursor_context("~r/ /") == {:operator, ~c"/"}
+      assert CF.cursor_context("~r/foo/") == {:local_arity, ~c"foo"}
+
       assert CF.cursor_context("~R") == {:sigil, ~c"R"}
       assert CF.cursor_context("~R/") == :none
       assert CF.cursor_context("~R<") == :none
@@ -365,6 +384,76 @@ defmodule CodeFragmentTest do
       assert CF.cursor_context("@hello_wo") == {:module_attribute, ~c"hello_wo"}
     end
 
+    test "keyword or binary operator" do
+      # Literals
+      assert CF.cursor_context("Foo.Bar ") == {:block_keyword_or_binary_operator, ~c""}
+      assert CF.cursor_context("Foo ") == {:block_keyword_or_binary_operator, ~c""}
+      assert CF.cursor_context(":foo ") == {:block_keyword_or_binary_operator, ~c""}
+      assert CF.cursor_context("123 ") == {:block_keyword_or_binary_operator, ~c""}
+      assert CF.cursor_context("nil ") == {:block_keyword_or_binary_operator, ~c""}
+      assert CF.cursor_context("true ") == {:block_keyword_or_binary_operator, ~c""}
+      assert CF.cursor_context("false ") == {:block_keyword_or_binary_operator, ~c""}
+      assert CF.cursor_context("\"foo\" ") == {:block_keyword_or_binary_operator, ~c""}
+      assert CF.cursor_context("'foo' ") == {:block_keyword_or_binary_operator, ~c""}
+
+      # Containers
+      assert CF.cursor_context("(foo) ") == {:block_keyword_or_binary_operator, ~c""}
+      assert CF.cursor_context("[foo] ") == {:block_keyword_or_binary_operator, ~c""}
+      assert CF.cursor_context("{foo} ") == {:block_keyword_or_binary_operator, ~c""}
+      assert CF.cursor_context("<<foo>> ") == {:block_keyword_or_binary_operator, ~c""}
+
+      # False positives
+      assert CF.cursor_context("foo ~>> ") == {:operator_call, ~c"~>>"}
+      assert CF.cursor_context("foo >>> ") == {:operator_call, ~c">>>"}
+    end
+
+    test "keyword from keyword or binary operator" do
+      # Literals
+      assert CF.cursor_context("Foo.Bar do") == {:block_keyword_or_binary_operator, ~c"do"}
+      assert CF.cursor_context("Foo.Bar d") == {:block_keyword_or_binary_operator, ~c"d"}
+      assert CF.cursor_context("Foo d") == {:block_keyword_or_binary_operator, ~c"d"}
+      assert CF.cursor_context(":foo d") == {:block_keyword_or_binary_operator, ~c"d"}
+      assert CF.cursor_context("123 d") == {:block_keyword_or_binary_operator, ~c"d"}
+      assert CF.cursor_context("nil d") == {:block_keyword_or_binary_operator, ~c"d"}
+      assert CF.cursor_context("true d") == {:block_keyword_or_binary_operator, ~c"d"}
+      assert CF.cursor_context("false d") == {:block_keyword_or_binary_operator, ~c"d"}
+      assert CF.cursor_context("\"foo\" d") == {:block_keyword_or_binary_operator, ~c"d"}
+      assert CF.cursor_context("'foo' d") == {:block_keyword_or_binary_operator, ~c"d"}
+
+      # Containers
+      assert CF.cursor_context("(foo) d") == {:block_keyword_or_binary_operator, ~c"d"}
+      assert CF.cursor_context("[foo] d") == {:block_keyword_or_binary_operator, ~c"d"}
+      assert CF.cursor_context("{foo} d") == {:block_keyword_or_binary_operator, ~c"d"}
+      assert CF.cursor_context("<<foo>> d") == {:block_keyword_or_binary_operator, ~c"d"}
+
+      # False positives
+      assert CF.cursor_context("foo ~>> d") == {:local_or_var, ~c"d"}
+      assert CF.cursor_context("foo >>> d") == {:local_or_var, ~c"d"}
+    end
+
+    test "operator from keyword or binary operator" do
+      # Literals
+      assert CF.cursor_context("Foo.Bar +") == {:operator, ~c"+"}
+      assert CF.cursor_context("Foo +") == {:operator, ~c"+"}
+      assert CF.cursor_context(":foo +") == {:operator, ~c"+"}
+      assert CF.cursor_context("123 +") == {:operator, ~c"+"}
+      assert CF.cursor_context("nil +") == {:operator, ~c"+"}
+      assert CF.cursor_context("true +") == {:operator, ~c"+"}
+      assert CF.cursor_context("false +") == {:operator, ~c"+"}
+      assert CF.cursor_context("\"foo\" +") == {:operator, ~c"+"}
+      assert CF.cursor_context("'foo' +") == {:operator, ~c"+"}
+
+      # Containers
+      assert CF.cursor_context("(foo) +") == {:operator, ~c"+"}
+      assert CF.cursor_context("[foo] +") == {:operator, ~c"+"}
+      assert CF.cursor_context("{foo} +") == {:operator, ~c"+"}
+      assert CF.cursor_context("<<foo>> +") == {:operator, ~c"+"}
+
+      # False positives
+      assert CF.cursor_context("foo ~>> +") == {:operator, ~c"+"}
+      assert CF.cursor_context("foo >>> +") == {:operator, ~c"+"}
+    end
+
     test "none" do
       # Punctuation
       assert CF.cursor_context(")") == :none
@@ -396,8 +485,6 @@ defmodule CodeFragmentTest do
       assert CF.cursor_context("HelloWór") == :none
       assert CF.cursor_context("@Hello") == :none
       assert CF.cursor_context("Hello(") == :none
-      assert CF.cursor_context("Hello ") == :none
-      assert CF.cursor_context("hello.World") == {:alias, {:local_or_var, ~c"hello"}, ~c"World"}
 
       # Identifier
       assert CF.cursor_context("foo@bar") == :none
@@ -488,14 +575,20 @@ defmodule CodeFragmentTest do
       assert CF.surround_context("안녕_세상", {1, 6}) == :none
 
       # Keywords are not local or var
-      for keyword <- ~w(do end after catch else rescue fn true false nil)c do
-        keyword_length = length(keyword) + 1
-
-        assert %{
-                 context: {:keyword, ^keyword},
+      for keyword <- ~w(do end after catch else rescue fn true false nil)c,
+          length = length(keyword),
+          i <- 1..length do
+        assert CF.surround_context(keyword, {1, i}) == %{
+                 context: {:keyword, keyword},
                  begin: {1, 1},
-                 end: {1, ^keyword_length}
-               } = CF.surround_context(keyword, {1, 1})
+                 end: {1, length + 1}
+               }
+
+        assert CF.surround_context(~c"Foo " ++ keyword, {1, 4 + i}) == %{
+                 context: {:keyword, keyword},
+                 begin: {1, 5},
+                 end: {1, length + 5}
+               }
       end
     end
 
@@ -590,6 +683,12 @@ defmodule CodeFragmentTest do
                  begin: {1, 1},
                  end: {1, byte_size(op) + 1}
                }
+
+        assert CF.surround_context("Foo #{op}", {1, 4 + i}) == %{
+                 context: {:operator, String.to_charlist(op)},
+                 begin: {1, 5},
+                 end: {1, byte_size(op) + 5}
+               }
       end
     end
 
@@ -1315,9 +1414,15 @@ defmodule CodeFragmentTest do
       assert cc2q!("(fn x ->", trailing_fragment: ":ok end)") ==
                s2q!("(fn x -> __cursor__() end)")
 
-      assert cc2q!("(fn x ->", trailing_fragment: ":ok end)") ==
+      assert cc2q!("(fn x ->", trailing_fragment: "\n:ok end)") ==
                s2q!("(fn x -> __cursor__() end)")
 
+      assert cc2q!("(fn x when ", trailing_fragment: "-> :ok end)") ==
+               s2q!("(fn x when __cursor__() -> :ok end)")
+
+      assert cc2q!("(fn x when ", trailing_fragment: "->\n:ok end)") ==
+               s2q!("(fn x when __cursor__() -> :ok end)")
+
       assert cc2q!("(fn") == s2q!("(__cursor__())")
       assert cc2q!("(fn x") == s2q!("(__cursor__())")
       assert cc2q!("(fn x,") == s2q!("(__cursor__())")
@@ -1327,6 +1432,26 @@ defmodule CodeFragmentTest do
       assert cc2q!("(fn x, y -> x + y end") == s2q!("(__cursor__())")
     end
 
+    test "do -> end" do
+      assert cc2q!("if do\nx ->\n", trailing_fragment: "y\nz ->\nw\nend") ==
+               s2q!("if do\nx ->\n__cursor__()\nz -> \nw\nend")
+
+      assert cc2q!("if do\nx ->\ny", trailing_fragment: "\nz ->\nw\nend") ==
+               s2q!("if do\nx ->\n__cursor__()\nz -> \nw\nend")
+
+      assert cc2q!("if do\nx ->\ny\n", trailing_fragment: "\nz ->\nw\nend") ==
+               s2q!("if do\nx ->\ny\n__cursor__()\nz -> \nw\nend")
+
+      assert cc2q!("for x <- [], reduce: %{} do\ny, ", trailing_fragment: "-> :ok\nend") ==
+               s2q!("for x <- [], reduce: %{} do\ny, __cursor__() -> :ok\nend")
+
+      assert cc2q!("for x <- [], reduce: %{} do\ny, z when ", trailing_fragment: "-> :ok\nend") ==
+               s2q!("for x <- [], reduce: %{} do\ny, z when __cursor__() -> :ok\nend")
+
+      assert cc2q!("case do\na -> a\nb = ", trailing_fragment: "c -> c\nend") ==
+               s2q!("case do\na -> a\nb = __cursor__() -> c\nend")
+    end
+
     test "removes tokens until opening" do
       assert cc2q!("(123") == s2q!("(__cursor__())")
       assert cc2q!("[foo") == s2q!("[__cursor__()]")
