@@ -1,3 +1,7 @@
+# SPDX-License-Identifier: Apache-2.0
+# SPDX-FileCopyrightText: 2021 The Elixir Team
+# SPDX-FileCopyrightText: 2012 Plataformatec
+
 Code.require_file("test_helper.exs", __DIR__)
 
 defmodule Inspect.AtomTest do
@@ -131,6 +135,8 @@ defmodule Inspect.BitStringTest do
     assert inspect(" ゆんゆん") == "\" ゆんゆん\""
     # BOM
     assert inspect("\uFEFFhello world") == "\"\\uFEFFhello world\""
+    # Invisible characters
+    assert inspect("\u2063") == "\"\\u2063\""
   end
 
   test "infer" do
@@ -330,11 +336,17 @@ defmodule Inspect.ListTest do
   end
 
   test "nested" do
-    assert inspect(Enum.reduce(1..100, [0], &[&2, Integer.to_string(&1)]), limit: 5) ==
-             "[[[[[[...], ...], \"97\"], \"98\"], \"99\"], \"100\"]"
+    assert inspect(Enum.reduce(1..5, [0], &[&2, &1]), limit: 5) ==
+             "[[[[[[...], ...], ...], ...], ...], ...]"
 
-    assert inspect(Enum.reduce(1..100, [0], &[&2 | Integer.to_string(&1)]), limit: 5) ==
-             "[[[[[[...] | \"96\"] | \"97\"] | \"98\"] | \"99\"] | \"100\"]"
+    assert inspect(Enum.reduce(1..5, [0], &[&2, &1]), limit: 10) ==
+             "[[[[[[0], 1], 2], 3], 4], ...]"
+
+    assert inspect(Enum.reduce(1..6, [0], &[&2, &1]), limit: 10) ==
+             "[[[[[[[0], 1], 2], 3], ...], ...], ...]"
+
+    assert inspect(Enum.reduce(1..100, [0], &[&2 | &1]), limit: 5) ==
+             "[[[[[[...] | 96] | 97] | 98] | 99] | 100]"
   end
 
   test "codepoints" do
@@ -433,6 +445,13 @@ defmodule Inspect.MapTest do
              "%{__struct__: Inspect.MapTest.Public, foo: :bar, key: 1}"
   end
 
+  test "public modified struct with defimpl" do
+    map_set = MapSet.new([1, 2])
+
+    assert inspect(Map.put(map_set, :foo, :bar), custom_options: [sort_maps: true]) ==
+             "%{__struct__: MapSet, foo: :bar, map: %{1 => [], 2 => []}}"
+  end
+
   test "private struct" do
     assert inspect(%{__struct__: Private, key: 1}, custom_options: [sort_maps: true]) ==
              "%{__struct__: Inspect.MapTest.Private, key: 1}"
@@ -524,16 +543,20 @@ defmodule Inspect.MapTest do
     # Inspect.Error is raised here when we tried to print the error message
     # called by another exception (Protocol.UndefinedError in this case)
     exception_message = ~s'''
-    protocol Enumerable not implemented for #Inspect.Error<
-      got ArgumentError with message:
+    protocol Enumerable not implemented for Inspect.MapTest.Failing (a struct)
 
-          """
-          errors were found at the given arguments:
+    Got value:
 
-            * 1st argument: not an atom
-          """
+        #Inspect.Error<
+          got ArgumentError with message:
 
-      while inspecting:
+              """
+              errors were found at the given arguments:
+
+                * 1st argument: not an atom
+              """
+
+          while inspecting:
 
     '''
 
@@ -589,10 +612,9 @@ defmodule Inspect.MapTest do
 
     {my_argument_error, stacktrace} =
       try do
-        Atom.to_string(failing.name)
+        atom_to_string(Process.get(:unused, failing.name))
       rescue
-        e ->
-          {e, __STACKTRACE__}
+        e -> {e, __STACKTRACE__}
       end
 
     inspected =
@@ -608,6 +630,10 @@ defmodule Inspect.MapTest do
     assert inspected =~ message
   end
 
+  defp atom_to_string(atom) do
+    Atom.to_string(atom)
+  end
+
   test "exception" do
     assert inspect(%RuntimeError{message: "runtime error"}) ==
              "%RuntimeError{message: \"runtime error\"}"
@@ -702,6 +728,17 @@ defmodule Inspect.MapTest do
                  end
   end
 
+  test "passing a non-list to the :only option" do
+    assert_raise ArgumentError,
+                 "invalid value :not_a_list in :only when deriving the Inspect protocol for Inspect.MapTest.StructInvalidListInOnlyOption (expected a list)",
+                 fn ->
+                   defmodule StructInvalidListInOnlyOption do
+                     @derive {Inspect, only: :not_a_list}
+                     defstruct [:a, :b]
+                   end
+                 end
+  end
+
   defmodule StructWithExceptOption do
     @derive {Inspect, except: [:b, :c]}
     defstruct [:a, :b, :c, :d]
@@ -759,6 +796,20 @@ defmodule Inspect.MapTest do
     assert inspect(struct) ==
              "#Inspect.MapTest.StructWithExceptOptionalAndOrder<d: nil, a: nil, ...>"
   end
+
+  defmodule StructWithOptionalAll do
+    @derive {Inspect, optional: :all}
+    defstruct [:a, :b, :c, :d]
+  end
+
+  test "struct with :optional set to :all" do
+    struct = %StructWithOptionalAll{a: 1, b: 2}
+
+    assert inspect(struct) == "%Inspect.MapTest.StructWithOptionalAll{a: 1, b: 2}"
+
+    struct = %StructWithOptionalAll{}
+    assert inspect(struct) == "%Inspect.MapTest.StructWithOptionalAll{}"
+  end
 end
 
 defmodule Inspect.OthersTest do
@@ -795,8 +846,8 @@ defmodule Inspect.OthersTest do
     Application.put_env(:elixir, :anony, V.fun())
     Application.put_env(:elixir, :named, &V.fun/0)
 
-    :code.delete(V)
     :code.purge(V)
+    :code.delete(V)
 
     anony = Application.get_env(:elixir, :anony)
     named = Application.get_env(:elixir, :named)
@@ -920,7 +971,11 @@ defmodule Inspect.CustomProtocolTest do
     got Protocol.UndefinedError with message:
 
         """
-        protocol Inspect.CustomProtocolTest.CustomInspect not implemented for %Inspect.CustomProtocolTest.MissingImplementation{} of type Inspect.CustomProtocolTest.MissingImplementation (a struct)
+        protocol Inspect.CustomProtocolTest.CustomInspect not implemented for Inspect.CustomProtocolTest.MissingImplementation (a struct)
+
+        Got value:
+
+            %Inspect.CustomProtocolTest.MissingImplementation{}
         """
 
     while inspecting:
@@ -947,7 +1002,11 @@ defmodule Inspect.CustomProtocolTest do
       got Protocol.UndefinedError with message:
 
           """
-          protocol Inspect.CustomProtocolTest.CustomInspect not implemented for %Inspect.CustomProtocolTest.MissingImplementation{} of type Inspect.CustomProtocolTest.MissingImplementation (a struct)
+          protocol Inspect.CustomProtocolTest.CustomInspect not implemented for Inspect.CustomProtocolTest.MissingImplementation (a struct)
+
+          Got value:
+
+              %Inspect.CustomProtocolTest.MissingImplementation{}
           """
 
       while inspecting:
