@@ -1,3 +1,7 @@
+%% SPDX-License-Identifier: Apache-2.0
+%% SPDX-FileCopyrightText: 2021 The Elixir Team
+%% SPDX-FileCopyrightText: 2012 Plataformatec
+
 -module(function_test).
 -include_lib("eunit/include/eunit.hrl").
 
@@ -67,7 +71,7 @@ function_call_without_arg_test() ->
   {3, _} = eval("x = fn -> 2 + 1 end\nx.()").
 
 function_call_do_end_test() ->
-  {[1, [{do, 2}, {else, 3}]], _} = eval("x = fn a, b -> [a, b] end\nx.(1) do\n2\nelse 3\nend").
+  {[1, [{do, 2}, {'else', 3}]], _} = eval("x = fn a, b -> [a, b] end\nx.(1) do\n2\nelse 3\nend").
 
 function_call_with_assignment_test() ->
   {3, [{a, _}, {c, 3}]} = eval("a = fn x -> x + 2 end; c = a.(1)").
