@@ -1,11 +1,13 @@
+%% SPDX-License-Identifier: Apache-2.0
+%% SPDX-FileCopyrightText: 2021 The Elixir Team
+%% SPDX-FileCopyrightText: 2012 Plataformatec
+
 -module(test_helper).
 -export([test/0, run_and_remove/2, throw_elixir/1, throw_erlang/1]).
 -define(TESTS, [
   atom_test,
   control_test,
   function_test,
-  match_test,
-  operators_test,
   string_test,
   tokenizer_test
 ]).
@@ -29,7 +31,7 @@ run_and_remove(Fun, Modules) ->
 % Throws an error with the Erlang Abstract Form from the Elixir string
 throw_elixir(String) ->
   Forms = elixir:'string_to_quoted!'(String, 1, 1, <<"nofile">>, []),
-  {Expr, _, _} = elixir:quoted_to_erl(Forms, elixir:env_for_eval([])),
+  {Expr, _, _, _} = elixir:quoted_to_erl(Forms, elixir:env_for_eval([])),
   erlang:error(io:format("~p~n", [Expr])).
 
 % Throws an error with the Erlang Abstract Form from the Erlang string
