@@ -1,3 +1,7 @@
+# SPDX-License-Identifier: Apache-2.0
+# SPDX-FileCopyrightText: 2021 The Elixir Team
+# SPDX-FileCopyrightText: 2012 Plataformatec
+
 defmodule IEx.Config do
   @moduledoc false
   use Agent
@@ -9,12 +13,37 @@ defmodule IEx.Config do
     :inspect,
     :history_size,
     :default_prompt,
-    :continuation_prompt,
     :alive_prompt,
-    :alive_continuation_prompt,
-    :width
+    :width,
+    :parser,
+    :dot_iex,
+    :auto_reload
   ]
 
+  # Generate a continuation prompt based on IEx prompt.
+  # This is set as global configuration on app start.
+  def prompt(prompt) do
+    case Enum.split_while(prompt, &(&1 != ?()) do
+      # It is not the default Elixir shell, so we use the default prompt
+      {_, []} ->
+        List.duplicate(?\s, max(0, prompt_width(prompt) - 3)) ++ ~c".. "
+
+      {left, right} ->
+        List.duplicate(?., prompt_width(left)) ++ right
+    end
+  end
+
+  # TODO: Remove this when we require Erlang/OTP 27+
+  @compile {:no_warn_undefined, :prim_tty}
+  @compile {:no_warn_undefined, :shell}
+  defp prompt_width(prompt) do
+    if function_exported?(:prim_tty, :npwcwidthstring, 1) do
+      :prim_tty.npwcwidthstring(prompt)
+    else
+      :shell.prompt_width(prompt)
+    end
+  end
+
   # Read API
 
   def configuration() do
@@ -46,16 +75,14 @@ defmodule IEx.Config do
     Application.fetch_env!(:iex, :default_prompt)
   end
 
-  def continuation_prompt() do
-    Application.get_env(:iex, :continuation_prompt, default_prompt())
-  end
-
   def alive_prompt() do
     Application.fetch_env!(:iex, :alive_prompt)
   end
 
-  def alive_continuation_prompt() do
-    Application.get_env(:iex, :alive_continuation_prompt, alive_prompt())
+  def parser() do
+    # Since the parser itself can be invoked from the remote node
+    # without IEx running, we cannot use fetch_env!
+    Application.get_env(:iex, :parser, {IEx.Evaluator, :parse, []})
   end
 
   def color(color) do
@@ -65,11 +92,8 @@ defmodule IEx.Config do
   defp color(color, colors) do
     if colors_enabled?(colors) do
       case Keyword.fetch(colors, color) do
-        {:ok, value} ->
-          value
-
-        :error ->
-          default_color(color)
+        {:ok, value} -> value
+        :error -> default_color(color)
       end
     else
       nil
@@ -77,13 +101,15 @@ defmodule IEx.Config do
   end
 
   defp colors_enabled?(colors) do
-    case Keyword.fetch(colors, :enabled) do
-      {:ok, enabled} ->
-        enabled
+    Keyword.get_lazy(colors, :enabled, &IO.ANSI.enabled?/0)
+  end
 
-      :error ->
-        IO.ANSI.enabled?()
-    end
+  def dot_iex() do
+    Application.get_env(:iex, :dot_iex)
+  end
+
+  def auto_reload?() do
+    Application.fetch_env!(:iex, :auto_reload)
   end
 
   # Used by default on evaluation cycle
@@ -100,16 +126,7 @@ defmodule IEx.Config do
 
   # Used by inspect
   defp default_color(:syntax_colors) do
-    [
-      atom: :cyan,
-      string: :green,
-      list: :default_color,
-      boolean: :magenta,
-      nil: :magenta,
-      tuple: :default_color,
-      binary: :default_color,
-      map: :default_color
-    ]
+    IO.ANSI.syntax_colors()
   end
 
   # Used by ansi docs
@@ -195,10 +212,11 @@ defmodule IEx.Config do
   defp validate_option({:inspect, new}) when is_list(new), do: :ok
   defp validate_option({:history_size, new}) when is_integer(new), do: :ok
   defp validate_option({:default_prompt, new}) when is_binary(new), do: :ok
-  defp validate_option({:continuation_prompt, new}) when is_binary(new), do: :ok
   defp validate_option({:alive_prompt, new}) when is_binary(new), do: :ok
-  defp validate_option({:alive_continuation_prompt, new}) when is_binary(new), do: :ok
   defp validate_option({:width, new}) when is_integer(new), do: :ok
+  defp validate_option({:parser, tuple}) when tuple_size(tuple) == 3, do: :ok
+  defp validate_option({:dot_iex, path}) when is_binary(path), do: :ok
+  defp validate_option({:auto_reload, enabled}) when is_boolean(enabled), do: :ok
 
   defp validate_option(option) do
     raise ArgumentError, "invalid configuration #{inspect(option)}"
