@@ -1,3 +1,7 @@
+# SPDX-License-Identifier: Apache-2.0
+# SPDX-FileCopyrightText: 2021 The Elixir Team
+# SPDX-FileCopyrightText: 2012 Plataformatec
+
 defmodule IEx.Helpers do
   @moduledoc """
   Welcome to Interactive Elixir. You are currently
@@ -109,6 +113,7 @@ defmodule IEx.Helpers do
     * `:force` - when `true`, forces the application to recompile
 
   """
+  @spec recompile(force: boolean()) :: :ok | :error | :noop
   def recompile(options \\ []) do
     cond do
       not mix_started?() ->
@@ -170,11 +175,8 @@ defmodule IEx.Helpers do
   end
 
   defp reenable_tasks(config) do
-    Mix.Task.reenable("compile")
-    Mix.Task.reenable("compile.all")
-    Mix.Task.reenable("compile.protocols")
     compilers = config[:compilers] || Mix.compilers()
-    Enum.each(compilers, &Mix.Task.reenable("compile.#{&1}"))
+    Mix.Task.Compiler.reenable(compilers)
   end
 
   @doc """
@@ -337,7 +339,7 @@ defmodule IEx.Helpers do
   """
   defmacro open(term) do
     quote do
-      IEx.Introspection.open(unquote(IEx.Introspection.decompose(term, __CALLER__)))
+      IEx.Introspection.open(unquote(decompose(term, __CALLER__)))
     end
   end
 
@@ -366,7 +368,7 @@ defmodule IEx.Helpers do
   """
   defmacro h(term) do
     quote do
-      IEx.Introspection.h(unquote(IEx.Introspection.decompose(term, __CALLER__)))
+      IEx.Introspection.h(unquote(decompose(term, __CALLER__)))
     end
   end
 
@@ -385,7 +387,7 @@ defmodule IEx.Helpers do
   """
   defmacro b(term) do
     quote do
-      IEx.Introspection.b(unquote(IEx.Introspection.decompose(term, __CALLER__)))
+      IEx.Introspection.b(unquote(decompose(term, __CALLER__)))
     end
   end
 
@@ -410,7 +412,7 @@ defmodule IEx.Helpers do
   """
   defmacro t(term) do
     quote do
-      IEx.Introspection.t(unquote(IEx.Introspection.decompose(term, __CALLER__)))
+      IEx.Introspection.t(unquote(decompose(term, __CALLER__)))
     end
   end
 
@@ -498,7 +500,9 @@ defmodule IEx.Helpers do
 
     elixir =
       if elixir != [] do
-        {:ok, modules, _warning} = Kernel.ParallelCompiler.compile(elixir)
+        {:ok, modules, _warning} =
+          Kernel.ParallelCompiler.compile(elixir, return_diagnostics: true)
+
         modules
       else
         []
@@ -555,6 +559,13 @@ defmodule IEx.Helpers do
     dont_display_result()
   end
 
+  defp decompose(term, context) do
+    case IEx.Introspection.decompose(term, context) do
+      :error -> term
+      m_mf_mfa -> Macro.escape(m_mf_mfa)
+    end
+  end
+
   # Given any "term", this function returns all the protocols in
   # :code.get_path() implemented by the data structure of such term, in the form
   # of a binary like "Protocol1, Protocol2, Protocol3".
@@ -591,13 +602,13 @@ defmodule IEx.Helpers do
   def runtime_info(topic) when is_atom(topic) and topic in @runtime_info_topics do
     topic
     |> List.wrap()
-    |> runtime_info
+    |> runtime_info()
   end
 
   def runtime_info(topics) when is_list(topics) do
     topics
     |> Enum.uniq()
-    |> print_runtime_info
+    |> print_runtime_info()
   end
 
   defp print_runtime_info(topics) do
@@ -1024,6 +1035,9 @@ defmodule IEx.Helpers do
 
       {:error, :enoent} ->
         IO.puts(IEx.color(:eval_error, "No directory #{directory}"))
+
+      {:error, reason} ->
+        IO.puts(IEx.color(:eval_error, :file.format_error(reason)))
     end
 
     dont_display_result()
@@ -1068,6 +1082,9 @@ defmodule IEx.Helpers do
 
       {:error, :enotdir} ->
         IO.puts(IEx.color(:eval_info, Path.absname(path)))
+
+      {:error, reason} ->
+        IO.puts(IEx.color(:eval_error, :file.format_error(reason)))
     end
 
     dont_display_result()
@@ -1522,8 +1539,11 @@ defmodule IEx.Helpers do
     end
   end
 
-  defp compile_elixir(exs, :in_memory), do: Kernel.ParallelCompiler.compile(exs)
-  defp compile_elixir(exs, path), do: Kernel.ParallelCompiler.compile_to_path(exs, path)
+  defp compile_elixir(exs, :in_memory),
+    do: Kernel.ParallelCompiler.compile(exs, return_diagnostics: true)
+
+  defp compile_elixir(exs, path),
+    do: Kernel.ParallelCompiler.compile_to_path(exs, path, return_diagnostics: true)
 
   # Compiles and loads an Erlang source file, returns {module, binary}
   defp compile_erlang(source) do
