@@ -1,3 +1,7 @@
+# SPDX-License-Identifier: Apache-2.0
+# SPDX-FileCopyrightText: 2021 The Elixir Team
+# SPDX-FileCopyrightText: 2012 Plataformatec
+
 defmodule IEx.Pry do
   @moduledoc """
   The low-level API for prying sessions and setting up breakpoints.
@@ -700,7 +704,7 @@ defmodule IEx.Pry do
   end
 
   def dbg(ast, options, %Macro.Env{} = env) when is_list(options) do
-    options = quote(do: Keyword.put(unquote(options), :print_location, false))
+    options = Keyword.put(options, :print_location, false)
 
     quote do
       IEx.Pry.pry(binding(), __ENV__)
