@@ -1,10 +1,18 @@
+# SPDX-License-Identifier: Apache-2.0
+# SPDX-FileCopyrightText: 2021 The Elixir Team
+# SPDX-FileCopyrightText: 2012 Plataformatec
+
 {line_exclude, line_include} =
   if line = System.get_env("LINE"), do: {[:test], [line: line]}, else: {[], []}
 
+Code.require_file("../../elixir/scripts/cover_record.exs", __DIR__)
+CoverageRecorder.maybe_record("logger")
+
 ExUnit.start(
   trace: !!System.get_env("TRACE"),
   include: line_include,
-  exclude: line_exclude
+  exclude: line_exclude,
+  assert_receive_timeout: String.to_integer(System.get_env("ELIXIR_ASSERT_TIMEOUT", "300"))
 )
 
 defmodule Logger.Case do
@@ -22,7 +30,7 @@ defmodule Logger.Case do
   end
 
   def wait_for_handler(manager, handler) do
-    unless handler in :gen_event.which_handlers(manager) do
+    if handler not in :gen_event.which_handlers(manager) do
       Process.sleep(10)
       wait_for_handler(manager, handler)
     end
