@@ -1,3 +1,7 @@
+# SPDX-License-Identifier: Apache-2.0
+# SPDX-FileCopyrightText: 2021 The Elixir Team
+# SPDX-FileCopyrightText: 2012 Plataformatec
+
 defmodule Mix.Tasks.Release.Init do
   use Mix.Task
 
@@ -87,8 +91,17 @@ defmodule Mix.Tasks.Release.Init do
     #!/bin/sh
     set -e
 
-    SELF=$(readlink "$0" || true)
-    if [ -z "$SELF" ]; then SELF="$0"; fi
+    readlink_f () {
+      cd "$(dirname "$1")" > /dev/null
+      filename="$(basename "$1")"
+      if [ -h "$filename" ]; then
+        readlink_f "$(readlink "$filename")"
+      else
+        echo "$(pwd -P)/$filename"
+      fi
+    }
+
+    SELF=$(readlink_f "$0")
     RELEASE_ROOT="$(CDPATH='' cd "$(dirname "$SELF")/.." && pwd -P)"
     export RELEASE_ROOT
     RELEASE_NAME="${RELEASE_NAME:-"<%= @release.name %>"}"
@@ -189,7 +202,7 @@ defmodule Mix.Tasks.Release.Init do
         ;;
 
       start_iex)
-        start "iex" --werl
+        start "iex"
         ;;
 
       daemon)
@@ -218,7 +231,7 @@ defmodule Mix.Tasks.Release.Init do
 
       remote)
         exec "$REL_VSN_DIR/iex" \
-             --werl --hidden --cookie "$RELEASE_COOKIE" \
+             --hidden --cookie "$RELEASE_COOKIE" \
              $(release_distribution "rem-$(rand)-$RELEASE_NODE") \
              --boot "$REL_VSN_DIR/$RELEASE_BOOT_SCRIPT_CLEAN" \
              --boot-var RELEASE_LIB "$RELEASE_ROOT/lib" \
@@ -292,9 +305,10 @@ defmodule Mix.Tasks.Release.Init do
     setlocal enabledelayedexpansion
 
     pushd .
-    cd "%~dp0\.."
+    pushd "%~dp0\.."
     set RELEASE_ROOT=%cd%
     popd
+    popd
 
     if not defined RELEASE_NAME (set RELEASE_NAME=<%= @release.name %>)
     if not defined RELEASE_VSN (for /f "tokens=1,2" %%K in ('type "!RELEASE_ROOT!\releases\start_erl.data"') do (set ERTS_VSN=%%K) && (set RELEASE_VSN=%%L))
@@ -335,7 +349,7 @@ defmodule Mix.Tasks.Release.Init do
     )
 
     if "%~1" == "start" (set "REL_EXEC=elixir" && set "REL_EXTRA=--no-halt" && set "REL_GOTO=start")
-    if "%~1" == "start_iex" (set "REL_EXEC=iex" && set "REL_EXTRA=--werl" && set "REL_GOTO=start")
+    if "%~1" == "start_iex" (set "REL_EXEC=iex" && set "REL_GOTO=start")
     if "%~1" == "install" (set "REL_GOTO=install")
     if "%~1" == "eval" (
       if "%~2" == "" (
@@ -348,8 +362,8 @@ defmodule Mix.Tasks.Release.Init do
     if not "!REL_GOTO!" == "" (
       findstr "RUNTIME_CONFIG=true" "!RELEASE_SYS_CONFIG!.config" >nul 2>&1 && (
         set DEFAULT_SYS_CONFIG=!RELEASE_SYS_CONFIG!
-        for /f "skip=1" %%X in ('wmic os get localdatetime') do if not defined TIMESTAMP set TIMESTAMP=%%X
-        set RELEASE_SYS_CONFIG=!RELEASE_TMP!\!RELEASE_NAME!-!RELEASE_VSN!-!TIMESTAMP:~0,11!-!RANDOM!.runtime
+        set "TIMESTAMP=%TIME::=%"
+        set RELEASE_SYS_CONFIG=!RELEASE_TMP!\!RELEASE_NAME!-!RELEASE_VSN!-!TIMESTAMP!-!RANDOM!.runtime
         mkdir "!RELEASE_TMP!" >nul 2>&1
         copy /y "!DEFAULT_SYS_CONFIG!.config" "!RELEASE_SYS_CONFIG!.config" >nul || (
           echo Cannot start release because it could not write to "!RELEASE_SYS_CONFIG!.config"
@@ -438,7 +452,7 @@ defmodule Mix.Tasks.Release.Init do
     )
 
     "!REL_VSN_DIR!\iex.bat" ^
-      --werl --hidden --cookie "!RELEASE_COOKIE!" ^
+      --hidden --cookie "!RELEASE_COOKIE!" ^
       !RELEASE_DISTRIBUTION_FLAG! ^
       --boot "!REL_VSN_DIR!\!RELEASE_BOOT_SCRIPT_CLEAN!" ^
       --boot-var RELEASE_LIB "!RELEASE_ROOT!\lib" ^
