@@ -1,3 +1,7 @@
+# SPDX-License-Identifier: Apache-2.0
+# SPDX-FileCopyrightText: 2021 The Elixir Team
+# SPDX-FileCopyrightText: 2012 Plataformatec
+
 Code.require_file("../../test_helper.exs", __DIR__)
 
 defmodule Mix.Tasks.DepsTest do
@@ -57,7 +61,7 @@ defmodule Mix.Tasks.DepsTest do
     end
   end
 
-  defmodule RawRepoDep do
+  defmodule RawRepoDepApp do
     def project do
       [
         app: :raw_sample,
@@ -215,6 +219,38 @@ defmodule Mix.Tasks.DepsTest do
     end)
   end
 
+  test "compiles deps using os partitions" do
+    System.put_env("MIX_OS_DEPS_COMPILE_PARTITION_COUNT", "2")
+
+    in_fixture("deps_status", fn ->
+      File.write!("mix.exs", """
+      defmodule ParDepsApp do
+        use Mix.Project
+
+        def project do
+          [
+            app: :par_sample,
+            version: "0.1.0",
+            deps: [
+              {:raw_repo, "0.1.0", path: "custom/raw_repo"},
+              {:git_repo, "0.1.0", path: #{inspect(fixture_path("git_repo"))}}
+            ]
+          ]
+        end
+      end
+      """)
+
+      Mix.Project.in_project(:par_sample, ".", fn _ ->
+        output = ExUnit.CaptureIO.capture_io(fn -> Mix.Tasks.Deps.Compile.run([]) end)
+        assert output =~ ~r/\d> Generated git_repo app/
+        assert output =~ ~r/\d> Generated raw_repo app/
+        assert_received {:mix_shell, :info, ["mix deps.compile running across 2 OS processes"]}
+      end)
+    end)
+  after
+    System.delete_env("MIX_OS_DEPS_COMPILE_PARTITION_COUNT")
+  end
+
   test "doesn't compile any umbrella apps if --skip-umbrella-children is given" do
     in_fixture("umbrella_dep/deps/umbrella", fn ->
       Mix.Project.in_project(:umbrella, ".", fn _ ->
@@ -282,36 +318,6 @@ defmodule Mix.Tasks.DepsTest do
     end)
   end
 
-  test "does not load deps with --no-deps-loading" do
-    in_fixture("deps_status", fn ->
-      Mix.Project.push(SuccessfulDepsApp)
-
-      # Start from scratch!
-      File.rm_rf("_build")
-
-      Mix.Tasks.Deps.Compile.run([])
-      Mix.Tasks.Deps.Loadpaths.run([])
-      assert File.exists?("_build/dev/lib/ok/ebin/ok.app")
-      assert File.exists?("_build/dev/lib/ok/priv/sample")
-
-      Mix.Tasks.Compile.run([])
-      assert to_charlist(Path.expand("_build/dev/lib/ok/ebin/")) in :code.get_path()
-      assert File.exists?("_build/dev/lib/sample/ebin/sample.app")
-
-      # Remove the deps without build_path
-      Mix.ProjectStack.post_config(deps: [])
-      Mix.State.clear_cache()
-      Mix.Project.pop()
-      Mix.Project.push(SuccessfulDepsApp)
-      Code.delete_path("_build/dev/lib/ok/ebin")
-
-      Mix.Tasks.Deps.Loadpaths.run(["--no-deps-loading"])
-      refute to_charlist(Path.expand("_build/dev/lib/ok/ebin/")) in :code.get_path()
-      assert File.exists?("_build/dev/lib/ok/ebin/ok.app")
-      assert File.exists?("_build/dev/lib/sample/ebin/sample.app")
-    end)
-  end
-
   ## deps.unlock
 
   test "unlocks all deps", context do
@@ -440,7 +446,7 @@ defmodule Mix.Tasks.DepsTest do
 
   test "sets deps env to prod by default" do
     in_fixture("deps_status", fn ->
-      Mix.Project.push(RawRepoDep)
+      Mix.Project.push(RawRepoDepApp)
 
       Mix.Tasks.Deps.Update.run(["--all"])
       assert_received {:mix_shell, :info, [":raw_repo env is prod"]}
@@ -734,7 +740,7 @@ defmodule Mix.Tasks.DepsTest do
       Mix.Task.clear()
 
       msg =
-        "  the dependency was built with an out-of-date Elixir version, run \"mix deps.compile\""
+        "  the dependency was built with an out-of-date Erlang/Elixir version, run \"mix deps.compile\""
 
       Mix.Tasks.Deps.run([])
       assert_received {:mix_shell, :info, [^msg]}
@@ -777,7 +783,7 @@ defmodule Mix.Tasks.DepsTest do
 
   test "checks if compile env changed" do
     in_fixture("deps_status", fn ->
-      Mix.Project.push(RawRepoDep)
+      Mix.Project.push(RawRepoDepApp)
       Mix.Tasks.Deps.Loadpaths.run([])
       assert_receive {:mix_shell, :info, ["Generated raw_repo app"]}
       assert Application.spec(:raw_repo, :vsn)
@@ -792,7 +798,7 @@ defmodule Mix.Tasks.DepsTest do
       Application.unload(:raw_repo)
       Mix.ProjectStack.pop()
       Mix.Task.clear()
-      Mix.Project.push(RawRepoDep)
+      Mix.Project.push(RawRepoDepApp)
       purge([RawRepo])
       Mix.Tasks.Loadconfig.load_compile("config/config.exs")
 
