@@ -1,3 +1,7 @@
+# SPDX-License-Identifier: Apache-2.0
+# SPDX-FileCopyrightText: 2021 The Elixir Team
+# SPDX-FileCopyrightText: 2012 Plataformatec
+
 Code.require_file("../../test_helper.exs", __DIR__)
 
 defmodule Mix.Tasks.FormatTest do
@@ -275,6 +279,35 @@ defmodule Mix.Tasks.FormatTest do
     end)
   end
 
+  test "ignores expanded patterns in excludes from .formatter.exs", context do
+    in_tmp(context.test, fn ->
+      File.write!(".formatter.exs", """
+      [
+        inputs: ["{a,.b}.ex"],
+        excludes: [".*.{ex,exs}"]
+      ]
+      """)
+
+      File.write!("a.ex", """
+      foo bar
+      """)
+
+      File.write!(".b.ex", """
+      foo bar
+      """)
+
+      Mix.Tasks.Format.run([])
+
+      assert File.read!("a.ex") == """
+             foo(bar)
+             """
+
+      assert File.read!(".b.ex") == """
+             foo bar
+             """
+    end)
+  end
+
   defmodule Elixir.SigilWPlugin do
     @behaviour Mix.Tasks.Format
 
@@ -510,6 +543,31 @@ defmodule Mix.Tasks.FormatTest do
     end)
   end
 
+  test "customizes plugin loading", context do
+    in_tmp(context.test, fn ->
+      File.write!(".formatter.exs", """
+      [
+        inputs: ["a.ex"],
+        plugins: [UnknownPlugin],
+      ]
+      """)
+
+      File.write!("a.ex", """
+      foo bar baz
+      """)
+
+      assert_raise Mix.NoProjectError, fn ->
+        Mix.Tasks.Format.formatter_for_file("a.ex")
+      end
+
+      assert_raise Mix.Error, "Formatter plugin UnknownPlugin cannot be found", fn ->
+        Mix.Tasks.Format.formatter_for_file("a.ex", plugin_loader: fn plugins -> plugins end)
+      end
+
+      assert Mix.Tasks.Format.formatter_for_file("a.ex", plugin_loader: fn _plugins -> [] end)
+    end)
+  end
+
   test "uses extension plugins with --stdin-filename", context do
     in_tmp(context.test, fn ->
       File.write!(".formatter.exs", """
@@ -534,6 +592,18 @@ defmodule Mix.Tasks.FormatTest do
     end)
   end
 
+  test "respects the --migrate flag", context do
+    in_tmp(context.test, fn ->
+      File.write!("a.ex", "unless foo, do: 'bar'\n")
+
+      Mix.Tasks.Format.run(["a.ex"])
+      assert File.read!("a.ex") == "unless foo, do: 'bar'\n"
+
+      Mix.Tasks.Format.run(["a.ex", "--migrate"])
+      assert File.read!("a.ex") == "if !foo, do: ~c\"bar\"\n"
+    end)
+  end
+
   test "uses inputs and configuration from --dot-formatter", context do
     in_tmp(context.test, fn ->
       File.write!("custom_formatter.exs", """
@@ -571,7 +641,7 @@ defmodule Mix.Tasks.FormatTest do
 
       root = File.cwd!()
 
-      {formatter_function, _options} =
+      {formatter_function, options} =
         File.cd!("lib", fn ->
           Mix.Tasks.Format.formatter_for_file("lib/a.ex", root: root)
         end)
@@ -581,6 +651,8 @@ defmodule Mix.Tasks.FormatTest do
              """) == """
              foo bar(baz)
              """
+
+      assert options[:root] == root
     end)
   end
 
@@ -605,12 +677,14 @@ defmodule Mix.Tasks.FormatTest do
 
       # Should hit the formatter
       {formatter, formatter_opts} = Mix.Tasks.Format.formatter_for_file("lib/extra/a.ex")
-      assert Keyword.get(formatter_opts, :locals_without_parens) == [my_fun: 2]
+      assert formatter_opts[:root] == Path.expand("lib")
+      assert formatter_opts[:locals_without_parens] == [my_fun: 2]
       assert formatter.("my_fun 1, 2") == "my_fun 1, 2\n"
 
       # Another directory should not hit the formatter
       {formatter, formatter_opts} = Mix.Tasks.Format.formatter_for_file("test/a.ex")
-      assert Keyword.get(formatter_opts, :locals_without_parens) == []
+      assert formatter_opts[:root] == Path.expand(".")
+      assert formatter_opts[:locals_without_parens] == []
       assert formatter.("my_fun 1, 2") == "my_fun(1, 2)\n"
 
       File.write!("lib/a.ex", """
@@ -681,7 +755,14 @@ defmodule Mix.Tasks.FormatTest do
       File.touch!(manifest_path, {{2010, 1, 1}, {0, 0, 0}})
 
       {_, formatter_opts} = Mix.Tasks.Format.formatter_for_file("a.ex")
-      assert [my_fun: 2] = Keyword.get(formatter_opts, :locals_without_parens)
+      assert [my_fun: 2] = formatter_opts[:locals_without_parens]
+
+      # Check the deps_path option is respected
+      assert_raise Mix.Error, ~r"Unknown dependency :my_dep given to :import_deps", fn ->
+        # Let's check that the manifest gets updated if it's stale.
+        File.touch!(manifest_path, {{2010, 1, 1}, {0, 0, 0}})
+        Mix.Tasks.Format.formatter_for_file("a.ex", deps_paths: %{})
+      end
 
       Mix.Tasks.Format.run(["a.ex"])
       assert File.stat!(manifest_path).mtime > {{2010, 1, 1}, {0, 0, 0}}
@@ -766,7 +847,8 @@ defmodule Mix.Tasks.FormatTest do
       Mix.Tasks.Format.run([])
 
       {_, formatter_opts} = Mix.Tasks.Format.formatter_for_file("lib/extra/a.ex")
-      assert [other_fun: 1] = Keyword.get(formatter_opts, :locals_without_parens)
+      assert formatter_opts[:root] == Path.expand("lib/extra")
+      assert formatter_opts[:locals_without_parens] == [other_fun: 1]
 
       assert File.read!("lib/extra/a.ex") == """
              my_fun(:foo, :bar)
